import { Request, Response } from "express";
import MissionService from "../../../service/operation/missionService";
import MissionFactory from "../../../factory/admin/operation/mission";
import { DeleteMissionCommand } from "../../../command/operation/mission/missionCommand";
import MissionCommandHandler from "../../../command/operation/mission/missionCommandHandler";
import SocketServer from "../../../websocket";
import PermissionHelper from "../../../helpers/permissionHelper";
import moment from "moment";

/**
 * @description get all missions
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getAllMissions(req: Request, res: Response): Promise<any> {
  let offset = parseInt((req.query.offset as string) ?? "0");
  let count = parseInt((req.query.count as string) ?? "25");
  let noLimit = req.query.noLimit === "true";

  let maxVisInDays = PermissionHelper.canValue(req.permissions, "maxVisInDays", true);
  let maxVisByMissions = PermissionHelper.canValue(req.permissions, "maxVisByMissions", true);

  let [missions, total] = await MissionService.getAll({
    offset: maxVisByMissions != "" && parseInt(maxVisByMissions) <= 25 ? 0 : offset,
    count: maxVisByMissions != "" ? Math.min(parseInt(maxVisByMissions), (offset + 1) * count) : count,
    noLimit: maxVisByMissions != "" ? false : noLimit,
    laterThan: maxVisInDays != "" ? moment().subtract(parseInt(maxVisInDays), "days").toDate() : undefined,
  });

  res.json({
    missions: MissionFactory.mapToBaseShort(missions),
    total: total,
    offset: offset,
    count: count,
  });
}

/**
 * @description get mission by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getMissionById(req: Request, res: Response): Promise<any> {
  const missionId = req.params.id;
  let mission = await MissionService.getById(missionId);

  res.json(MissionFactory.mapToSingleShort(mission));
}

/**
 * @description create mission
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function createMission(req: Request, res: Response): Promise<any> {
  let missionId = await MissionCommandHandler.create();

  SocketServer.broadcastNewMission(missionId);

  res.status(200).send(missionId);
}

/**
 * @description delete mission by id
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function deleteMissionById(req: Request, res: Response): Promise<any> {
  const missionId = req.params.id;

  let deleteMission: DeleteMissionCommand = {
    id: missionId,
  };
  await MissionCommandHandler.delete(deleteMission);

  res.sendStatus(204);
}
